{
    "$graph": [
        {
            "class": "Workflow",
            "label": "sequence (bam or fastqs) to trimmed fastqs and HISAT alignments",
            "requirements": [
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SchemaDefRequirement",
                    "types": [
                        {
                            "type": "record",
                            "name": "#sequence_data.yml/sequence_data",
                            "label": "sequence data with readgroup information",
                            "fields": [
                                {
                                    "type": [
                                        "null",
                                        "string"
                                    ],
                                    "name": "#sequence_data.yml/sequence_data/readgroup"
                                },
                                {
                                    "type": [
                                        {
                                            "type": "record",
                                            "name": "#sequence_data.yml/sequence_data/sequence/bam",
                                            "fields": [
                                                {
                                                    "type": "File",
                                                    "name": "#sequence_data.yml/sequence_data/sequence/bam/bam"
                                                }
                                            ]
                                        },
                                        {
                                            "type": "record",
                                            "name": "#sequence_data.yml/sequence_data/sequence/fastqs",
                                            "fields": [
                                                {
                                                    "type": "File",
                                                    "name": "#sequence_data.yml/sequence_data/sequence/fastqs/fastq1"
                                                },
                                                {
                                                    "type": "File",
                                                    "name": "#sequence_data.yml/sequence_data/sequence/fastqs/fastq2"
                                                }
                                            ]
                                        }
                                    ],
                                    "name": "#sequence_data.yml/sequence_data/sequence"
                                }
                            ]
                        }
                    ]
                }
            ],
            "inputs": [
                {
                    "type": "int",
                    "id": "#main/adapter_min_overlap"
                },
                {
                    "type": "string",
                    "id": "#main/adapter_trim_end"
                },
                {
                    "type": "File",
                    "id": "#main/adapters"
                },
                {
                    "type": "int",
                    "id": "#main/max_uncalled"
                },
                {
                    "type": "int",
                    "id": "#main/min_readlength"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#main/read_group_fields"
                },
                {
                    "type": "string",
                    "id": "#main/read_group_id"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".1.ht2",
                        ".2.ht2",
                        ".3.ht2",
                        ".4.ht2",
                        ".5.ht2",
                        ".6.ht2",
                        ".7.ht2",
                        ".8.ht2"
                    ],
                    "id": "#main/reference_index"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#main/strand/first",
                                "#main/strand/second",
                                "#main/strand/unstranded"
                            ]
                        }
                    ],
                    "id": "#main/strand"
                },
                {
                    "type": "#sequence_data.yml/sequence_data",
                    "id": "#main/unaligned"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "id": "#main/unzip_fastqs"
                }
            ],
            "steps": [
                {
                    "run": "#hisat2_align.cwl",
                    "in": [
                        {
                            "source": "#main/trim_fastq/fastqs",
                            "valueFrom": "$(self[0])",
                            "id": "#main/hisat2_align/fastq1"
                        },
                        {
                            "source": "#main/trim_fastq/fastqs",
                            "valueFrom": "$(self[1])",
                            "id": "#main/hisat2_align/fastq2"
                        },
                        {
                            "source": "#main/read_group_fields",
                            "id": "#main/hisat2_align/read_group_fields"
                        },
                        {
                            "source": "#main/read_group_id",
                            "id": "#main/hisat2_align/read_group_id"
                        },
                        {
                            "source": "#main/reference_index",
                            "id": "#main/hisat2_align/reference_index"
                        },
                        {
                            "source": "#main/strand",
                            "id": "#main/hisat2_align/strand"
                        }
                    ],
                    "out": [
                        "#main/hisat2_align/aligned_bam"
                    ],
                    "id": "#main/hisat2_align"
                },
                {
                    "run": "#sequence_to_fastq.cwl",
                    "in": [
                        {
                            "source": "#main/unaligned",
                            "valueFrom": "$(self.sequence.hasOwnProperty('bam')? self.sequence.bam : null)",
                            "id": "#main/sequence_to_fastq/bam"
                        },
                        {
                            "source": "#main/unaligned",
                            "valueFrom": "$(self.sequence.hasOwnProperty('fastq1')? self.sequence.fastq1 : null)",
                            "id": "#main/sequence_to_fastq/fastq1"
                        },
                        {
                            "source": "#main/unaligned",
                            "valueFrom": "$(self.sequence.hasOwnProperty('fastq2')? self.sequence.fastq2 : null)",
                            "id": "#main/sequence_to_fastq/fastq2"
                        },
                        {
                            "source": "#main/unzip_fastqs",
                            "id": "#main/sequence_to_fastq/unzip_fastqs"
                        }
                    ],
                    "out": [
                        "#main/sequence_to_fastq/fastq1",
                        "#main/sequence_to_fastq/fastq2"
                    ],
                    "id": "#main/sequence_to_fastq"
                },
                {
                    "run": "#trim_fastq.cwl",
                    "in": [
                        {
                            "source": "#main/adapter_min_overlap",
                            "id": "#main/trim_fastq/adapter_min_overlap"
                        },
                        {
                            "source": "#main/adapter_trim_end",
                            "id": "#main/trim_fastq/adapter_trim_end"
                        },
                        {
                            "source": "#main/adapters",
                            "id": "#main/trim_fastq/adapters"
                        },
                        {
                            "source": "#main/max_uncalled",
                            "id": "#main/trim_fastq/max_uncalled"
                        },
                        {
                            "source": "#main/min_readlength",
                            "id": "#main/trim_fastq/min_readlength"
                        },
                        {
                            "source": "#main/sequence_to_fastq/fastq1",
                            "id": "#main/trim_fastq/reads1"
                        },
                        {
                            "source": "#main/sequence_to_fastq/fastq2",
                            "id": "#main/trim_fastq/reads2"
                        }
                    ],
                    "out": [
                        "#main/trim_fastq/fastqs"
                    ],
                    "id": "#main/trim_fastq"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/hisat2_align/aligned_bam",
                    "id": "#main/aligned_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/trim_fastq/fastqs",
                    "id": "#main/fastqs"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "label": "HISAT2: align",
            "baseCommand": [
                "/usr/local/bin/hisat2"
            ],
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 16000,
                    "coresMin": 16
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "mgibio/hisat2-sambamba:0.1"
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "arguments": [
                "-p",
                "$(runtime.cores)",
                "--dta",
                {
                    "shellQuote": false,
                    "valueFrom": "|"
                },
                "/usr/local/bin/sambamba",
                "view",
                "-S",
                "-f",
                "bam",
                "-l",
                "0",
                "/dev/stdin",
                {
                    "shellQuote": false,
                    "valueFrom": "|"
                },
                "/usr/local/bin/sambamba",
                "sort",
                "-t",
                "$(runtime.cores)",
                "-m",
                "8G",
                "-o",
                "$(runtime.outdir)/aligned.bam",
                "/dev/stdin"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-1",
                        "position": -2
                    },
                    "id": "#hisat2_align.cwl/fastq1"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-2",
                        "position": -1
                    },
                    "id": "#hisat2_align.cwl/fastq2"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "--rg"
                        }
                    },
                    "inputBinding": {
                        "position": -4
                    },
                    "id": "#hisat2_align.cwl/read_group_fields"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--rg-id",
                        "position": -5
                    },
                    "id": "#hisat2_align.cwl/read_group_id"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".1.ht2",
                        ".2.ht2",
                        ".3.ht2",
                        ".4.ht2",
                        ".5.ht2",
                        ".6.ht2",
                        ".7.ht2",
                        ".8.ht2"
                    ],
                    "inputBinding": {
                        "prefix": "-x",
                        "position": -3
                    },
                    "id": "#hisat2_align.cwl/reference_index"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#hisat2_align.cwl/strand/first",
                                "#hisat2_align.cwl/strand/second",
                                "#hisat2_align.cwl/strand/unstranded"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "valueFrom": "${\n    if (inputs.strand) {\n        if (inputs.strand == 'first') {\n            return ['--rna-strandness RF'];\n        } else if (inputs.strand == 'second') {\n            return ['--rna-strandness FR'];\n        } else {\n            return [];\n        }\n    } else {\n            return []\n    }\n}\n",
                        "position": -6
                    },
                    "id": "#hisat2_align.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "aligned.bam"
                    },
                    "id": "#hisat2_align.cwl/aligned_bam"
                }
            ],
            "id": "#hisat2_align.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "Picard: BAM to FASTQ",
            "baseCommand": [
                "/bin/bash",
                "makefastqs.sh"
            ],
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 16000,
                    "tmpdirMin": 25000
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/picard:2.23.6"
                },
                {
                    "class": "SchemaDefRequirement",
                    "types": [
                        {
                            "$import": "#sequence_data.yml/sequence_data"
                        }
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "makefastqs.sh",
                            "entry": "set -o pipefail\nset -o errexit\nset -o nounset\n\nUNZIP=false\n\nwhile getopts \":b:1:2:d:u\" opt; do\n    case \"$opt\" in\n        b)\n            MODE=bam\n            BAM=\"$OPTARG\"\n            ;;\n        1)\n            MODE=fastq\n            FASTQ1=\"$OPTARG\"\n            ;;\n        2)  \n            MODE=fastq\n            FASTQ2=\"$OPTARG\"\n            ;;\n        d)\n            OUTDIR=\"$OPTARG\"\n            ;;\n        u)\n            UNZIP=true\n            ;;\n    esac\ndone\n\nif [[ \"$MODE\" == 'fastq' ]]; then #must be fastq input\n\n    #basename is a no-op if suffix isn't found,\n    #so it's safe to use on optional file extensions\n    unzip1=`basename \"$FASTQ1\" .gz`\n    unzip2=`basename \"$FASTQ2\" .gz`\n    nameroot1=`basename \"$unzip1\" .fastq`\n    nameroot2=`basename \"$unzip2\" .fastq`\n    nameroot1=`basename \"$nameroot1\" .fq`\n    nameroot2=`basename \"$nameroot2\" .fq`\n\n    if $UNZIP; then\n        if gzip -t $FASTQ1 2> /dev/null; then\n            gunzip -c $FASTQ1 > $OUTDIR/\"$nameroot1\".read1.fastq\n        else\n            cp $FASTQ1 $OUTDIR/\"$nameroot1\".read1.fastq\n        fi\n\n        if gzip -t $FASTQ2 2> /dev/null; then\n            gunzip -c $FASTQ2 > $OUTDIR/\"$nameroot2\".read2.fastq\n        else\n            cp $FASTQ2 $OUTDIR/\"$nameroot2\".read2.fastq\n        fi\n    else\n        if gzip -t $FASTQ1 2> /dev/null; then\n            cp $FASTQ1 $OUTDIR/\"$nameroot1\".read1.fastq.gz\n        else\n            cp $FASTQ1 $OUTDIR/\"$nameroot1\".read1.fastq\n        fi\n\n        if gzip -t $FASTQ2 2> /dev/null; then\n            cp $FASTQ2 $OUTDIR/\"$nameroot2\".read2.fastq.gz\n        else\n            cp $FASTQ2 $OUTDIR/\"$nameroot2\".read2.fastq\n        fi\n    fi\n\nelse # then\n    ##run samtofastq here, dumping to the same filenames\n    ## input file is $BAM\n\n    nameroot=`basename \"$BAM\" .bam`\n    /usr/bin/java -Xmx4g -jar /usr/picard/picard.jar SamToFastq I=\"$BAM\" INCLUDE_NON_PF_READS=true F=$OUTDIR/\"$nameroot\".read1.fastq F2=$OUTDIR/\"$nameroot\".read2.fastq VALIDATION_STRINGENCY=SILENT\nfi\n"
                        }
                    ]
                }
            ],
            "arguments": [
                {
                    "valueFrom": "$(runtime.outdir)",
                    "position": -6,
                    "prefix": "-d"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "-b"
                    },
                    "id": "#sequence_to_fastq.cwl/bam"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "-1"
                    },
                    "id": "#sequence_to_fastq.cwl/fastq1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "-2"
                    },
                    "id": "#sequence_to_fastq.cwl/fastq2"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "-u"
                    },
                    "doc": "If true, and the input sequence contains gzipped fastqs, they will be unzipped",
                    "id": "#sequence_to_fastq.cwl/unzip_fastqs"
                }
            ],
            "outputs": [
                {
                    "$import": "#sequence_to_fastq.cwl/fastq1"
                },
                {
                    "$import": "#sequence_to_fastq.cwl/fastq2"
                }
            ],
            "id": "#sequence_to_fastq.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "Trim FASTQ (flexbar)",
            "baseCommand": [
                "/opt/flexbar/flexbar"
            ],
            "arguments": [
                "--target",
                {
                    "valueFrom": "$(runtime.outdir)/$(inputs.reads1.nameroot).trimmed_read"
                },
                "--threads",
                {
                    "valueFrom": "$(runtime.cores)"
                }
            ],
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "ramMin": 16000,
                    "tmpdirMin": 25000,
                    "coresMin": 4
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "mgibio/bisulfite:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": "int",
                    "inputBinding": {
                        "prefix": "--adapter-min-overlap",
                        "position": 3
                    },
                    "id": "#trim_fastq.cwl/adapter_min_overlap"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--adapter-trim-end",
                        "position": 2
                    },
                    "id": "#trim_fastq.cwl/adapter_trim_end"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--adapters",
                        "position": 1
                    },
                    "id": "#trim_fastq.cwl/adapters"
                },
                {
                    "type": "int",
                    "inputBinding": {
                        "prefix": "--max-uncalled",
                        "position": 5
                    },
                    "id": "#trim_fastq.cwl/max_uncalled"
                },
                {
                    "type": "int",
                    "inputBinding": {
                        "prefix": "--min-read-length",
                        "position": 6
                    },
                    "id": "#trim_fastq.cwl/min_readlength"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--reads",
                        "position": 7
                    },
                    "id": "#trim_fastq.cwl/reads1"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--reads2",
                        "position": 8
                    },
                    "id": "#trim_fastq.cwl/reads2"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.reads1.nameroot).trimmed_read_1.fastq"
                    },
                    "id": "#trim_fastq.cwl/fastq1"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.reads1.nameroot).trimmed_read_2.fastq"
                    },
                    "id": "#trim_fastq.cwl/fastq2"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "$(inputs.reads1.nameroot).trimmed_read_*.fastq"
                    },
                    "id": "#trim_fastq.cwl/fastqs"
                }
            ],
            "id": "#trim_fastq.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
